######################################################################################
# R code
# File: logpost.r
#
# Example 17 in the paper by Amendola and Francq (2009):
# STAR model (equation (47)):
#
# Y_{t} = a0 +a1Y_{t-1} + [(b0-a0)+(b1-a1)Y_{t-1}]/[1 + exp{-gamma(Y_{t-1}-c)}] + e_{t},
#
#    where a0 = b0 = c = 0, a1 = -0.95, b1 = 0.9, gamma = 5, and 
#    e_{t} = i.i.d. with 0.9 N(0,0.5^{2}) + 0.05 N(3,1) + 0.05 N(-3,1).
#
# Generate a Metropolis algorithm with the random walk 
# kernel transition Q(x,y) ~ N(x,scale I_{6}), and with the 
# output of the lstar() as starting value \theta^{(0)}.
#
# STEPS:
# 1) Load R-tsDyn package and run the lstar(.) function
# # Fit a STAR model to the time series y
#   > fittedstar <- lstar(y,m=1,thDelay=0,control=list(maxit=3000))
#   > names(fittedstar$coefficients) <- c("a0","a1","b0","b1","gamma","c")
#   > summary(fittedstar)
# 
# Non linear autoregressive model
# LSTAR model
# Coefficients:
# Low regime:
#     const1     phi1.1 
# -0.2124685 -1.0995471 
#
# High regime:
#    const2    phi2.1 
# 0.1856542 1.9978530 
#
# Smoothing parameter: gamma = 100 
#
# Threshold
# Variable: Z(t) = + (1) X(t) 
#
# Value: 0.1782 
# 
# Residuals:
#        Min         1Q     Median         3Q        Max 
# -1.0834188 -0.2985737  0.0036982  0.2753107  1.2762891 
#
# Fit:
# residuals variance = 0.1967,  AIC = -313, MAPE = 112.3%
#
# Coefficient(s):
#        Estimate  Std. Error  t value  Pr(>|z|)    
# a0     -0.212469    0.087262  -2.4348    0.0149 *  
# a1     -1.099547    0.202777  -5.4224 5.879e-08 ***
# b0      0.185654    0.116974   1.5871    0.1125    
# b1      1.997853    0.217741   9.1753 < 2.2e-16 ***
# gamma 100.000057   97.908153   1.0214    0.3071     
# c       0.178170    0.023242   7.6659 1.776e-14 ***
# ---
# Signif. codes:  0 *** 0.001 ** 0.01 * 0.05 . 0.1   1 
#
# Non-linearity test of full-order LSTAR model against full-order AR model
#  F = 36.78 ; p-value = 6.688e-09 
#
# Threshold 
# Variable: Z(t) = + (1) X(t) 
#
# 2) The function metrop(.) of the R-mcmc package (and also the function
#    MCMCmetrop1R(.) of the R-MCMCpack package) can be employed to 
#    perform a Bayesian analysis with y = ts(y,freq=1).
#
# Reference:
# Amendola, A. and Francq, C. (2009).
#   Concepts and tools for nonlinear time series modelling.
#   MPRA Paper No. 15140. Available at 
#   htttp://mpra.ub.uni-muenchen.de/15140/, and for a more
#   recent version htttp://mpra.ub.uni-muenchen.de/16668/
#####################################################################################

theta.init0 <- {as.vector(c(fittedstar$coefficients[1:5],var(residuals(fittedstar),na.rm=T)))};

# "Burn-in" values with 3,000 simulations 
mcmcresults <- metrop(logpost, theta.init0, 3000, scale=0.02, y=y);
mcmcresults$accept
chaine                <- ts(mcmcresults$batch);
dimnames(chaine)[[2]] <- c("a0","a1","b0","b1","gamma","sigma2");
plot(chaine,main="Markov chain simulated by Metropolis")
 
#  Generating a longer trajectory of 30,000 values, and tuning the scale parameter 
mcmcresults <- {metrop(mcmcresults,nbatch=30000,scale=c(0.03,0.03,0.03,0.03,0.4,0.02),y=y)}
chaine                <- ts(mcmcresults$batch);
dimnames(chaine)[[2]] <- c("a0","a1","b0","b1","gamma","sigma2");
plot(chaine,main="Markov chain simulated by Metropolis nbatch=30,000")